import platform

from halo import Halo


def get_spinner_object():
    spinner_type = "dots"
    if platform.system() == "Windows":
        # Avoid unicode char issue
        spinner_type = "line"
    return Halo(text="Loading", spinner=spinner_type, color=None)


def spin(func):
    """
   Wrap Halo within a decorator then it's easy to disable it.
   """

    def inner(self, *args, **kwargs):
        with get_spinner_object() as spinner:
            func(self, spinner, *args, **kwargs)

    return inner



"""
Check for yogit update
"""
from functools import cmp_to_key
from packaging import version

import click

from yogit import get_version
from yogit.api.queries import RESTQuery
from yogit.yogit.logger import LOGGER

YOGIT_TAG_LIST_ENDPOINT = "/repos/hasboeuf/yogit/tags"


def compare(version1, version2):
    """
    Compare two X.Y.Z version string, can be used in sorted() function
    """
    if version1 == version2:
        return 0
    return -1 if version.parse(version1) < version.parse(version2) else 1


class YogitTagsQuery(RESTQuery):
    """
    Fetch tags from yogit repository
    """

    def __init__(self):
        super().__init__("/repos/hasboeuf/yogit/tags")
        self.tags = []

    def _handle_response(self, response):
        self.tags = [x["name"] for x in response]

    def get_tags(self):
        """
        Return fetched tags
        """
        return self.tags


class UpdateChecker:
    """
    Check is yogit is updatable
    """

    def __init__(self):
        self.query = YogitTagsQuery()

    def _is_outdated(self):
        current_version = get_version()
        try:
            self.query.execute()
            tags = sorted(self.query.get_tags(), key=cmp_to_key(compare), reverse=True)
            latest_version = tags[0]
            outdated = version.parse(current_version) < version.parse(latest_version)
        except Exception as exception:
            # Update check should never fail
            LOGGER.error(str(exception))
            return False, current_version, current_version
        return outdated, current_version, latest_version

    def check(self):
        """
        Compare current version with the latest one and print a message if needed
        """
        outdated, current_version, latest_version = self._is_outdated()
        if not outdated:
            return
        click.secho(
            (
                "You are using yogit {}, however version {} is available 🎁\n"
                "You can upgrade it with `pip3 install --upgrade yogit` 🚀\n"
                "Changelog: https://github.com/hasboeuf/yogit/blob/{}/CHANGELOG.md"
            ).format(current_version, latest_version, latest_version),
            fg="yellow",
        )



"""
Application logger
"""

import logging
import os
import sys

from yogit import get_name, get_version
from yogit.yogit.paths import get_log_path, SETTINGS_DIR


def get_logger(stdout=False, logger_name=get_name(), version=get_version()):
    """
    Create and configure a logger using a given name.
    """
    os.makedirs(SETTINGS_DIR, exist_ok=True)

    application_str = logger_name
    if version:
        application_str += " " + version
    formatter = logging.Formatter(
        fmt=(
            "%(asctime)s "
            "[{application}:%(process)d] "
            "[%(levelname)s] "
            "%(message)s".format(application=application_str)
        ),
        datefmt="%Y-%m-%dT%H:%M:%S%z",
    )

    file_log_handler = logging.FileHandler(get_log_path())
    file_log_handler.setLevel(logging.DEBUG)
    file_log_handler.setFormatter(formatter)

    local_logger = logging.getLogger(logger_name)
    local_logger.setLevel(logging.DEBUG)
    local_logger.addHandler(file_log_handler)

    if stdout:
        console_log_handler = logging.StreamHandler(sys.stdout)
        console_log_handler.setLevel(logging.DEBUG)
        console_log_handler.setFormatter(formatter)
        local_logger.addHandler(console_log_handler)

    return local_logger


LOGGER = get_logger()


def enable_stdout():
    """
    Prints logs in stdout
    """
    global LOGGER  # pylint: disable=global-statement
    LOGGER = get_logger(stdout=True)



"""
GraphQL queries used by yogit
"""
import textwrap

import click
from tabulate import tabulate

import yogit.api.statements as S
from yogit.api.client import GraphQLClient, RESTClient
from yogit.api.statement import prepare, prepare_pagination
from yogit.utils.dateutils import dt_for_str, days_ago_str
from yogit.utils.spinner import spin


def shorten_str(full_string):
    """
    Shorten string to 50 chars max, including an ending ellipsis
    Words are not truncated
    """
    return textwrap.shorten(full_string, width=50, placeholder="...")


class Query:
    """ Represent a GitHub query """

    def __init__(self):
        self._response = []
        self.client = None

    def _handle_response(self, response):
        self._response.append(response)

    def execute(self):
        """ Execute the query """
        raise NotImplementedError()

    def tabulate(self):
        """ Return tabulated result """
        raise NotImplementedError()

    def print(self):
        """ Print result """
        click.echo(self._response)


class GraphQLQuery(Query):
    def __init__(self, statement, variables=[], extra_data={}, pagination_offset=None):
        super().__init__()
        self.client = GraphQLClient()
        self.statement = statement
        self.variables = variables
        self.extra_data = extra_data
        self.pagination_offset = pagination_offset

    def get_pagination_info(self):
        raise NotImplementedError()

    def get_count(self):
        raise NotImplementedError()

    @spin
    def execute(self, spinner):
        prepared_statement = prepare(self.statement, self.variables, self.extra_data)
        if self.pagination_offset is None:
            response = self.client.get(prepared_statement)
            super()._handle_response(response)
            self._handle_response(response)
            return

        cursor = None
        has_next = True
        while has_next:
            paginated_statement = prepare_pagination(prepared_statement, self.pagination_offset, cursor)
            response = self.client.get(paginated_statement)
            super()._handle_response(response)
            self._handle_response(response)
            count = self.get_count()
            if count > 0:
                spinner.text = "Loading... (yet {} entries found)".format(count)
            pagination_info = self.get_pagination_info(response)
            has_next = pagination_info["hasNextPage"]
            cursor = pagination_info["endCursor"]


class RESTQuery(Query):
    def __init__(self, endpoint):
        super().__init__()
        self.client = RESTClient()
        self.endpoint = endpoint

    @spin
    def execute(self, spinner):
        response = self.client.get(self.endpoint)
        super()._handle_response(response)
        self._handle_response(response)


class LoginQuery(GraphQLQuery):
    def __init__(self):
        super().__init__(S.LOGIN_STATEMENT)
        self.login = None

    def _handle_response(self, response):
        self.login = response["data"]["viewer"]["login"]

    def get_login(self):
        return self.login


class ReviewRequestedQuery(GraphQLQuery):
    def __init__(self):
        super().__init__(S.REVIEW_REQUESTED_STATEMENT, [S.LOGIN_VARIABLE])
        self.data = []

    def _handle_response(self, response):
        for pr in response["data"]["search"]["edges"]:
            repo = pr["node"]["repository"]["nameWithOwner"]
            url = pr["node"]["url"]
            self.data.append([repo, url])
        self.data = sorted(self.data, key=lambda x: x[0])

    def print(self):
        if len(self.data) == 0:
            click.secho("All done! 🎉✨", bold=True)
        else:
            click.echo(tabulate(self.data, headers=["REPO", "URL"]))
            click.secho("Count: {}".format(len(self.data)), bold=True)


class ReviewListQuery(GraphQLQuery):
    def __init__(self):
        super().__init__(S.REVIEW_LIST_STATEMENT, pagination_offset=100)
        self.data = []

    def get_pagination_info(self, response):
        return response["data"]["viewer"]["contributionsCollection"]["pullRequestReviewContributions"]["pageInfo"]

    def get_count(self):
        return len(self.data)

    def _handle_response(self, response):
        for review in response["data"]["viewer"]["contributionsCollection"]["pullRequestReviewContributions"]["edges"]:
            pr_state = review["node"]["pullRequest"]["state"]
            if pr_state != "OPEN":
                continue

            url = review["node"]["pullRequest"]["url"]
            rv_state = review["node"]["pullRequestReview"]["state"]
            last_commit_pushed = dt_for_str(
                review["node"]["pullRequest"]["commits"]["edges"][0]["node"]["commit"]["pushedDate"]
            )

            rv_updated = review["node"]["pullRequestReview"]["updatedAt"]
            if rv_updated is None:
                rv_updated = review["node"]["pullRequestReview"]["createdAt"]
            rv_updated = dt_for_str(rv_updated)

            up_to_date = rv_updated > last_commit_pushed
            rv_updated_str = days_ago_str(rv_updated.date())

            rv_state_str = rv_state
            if not up_to_date:
                rv_state_str += " (new commits)"

            self.data.append([rv_updated.date(), rv_updated_str, url, rv_state_str])

        # Sort by url, then by reversed date:
        self.data = sorted(self.data, key=lambda x: x[2])
        self.data = sorted(self.data, key=lambda x: x[0], reverse=True)

    def print(self):
        if len(self.data) == 0:
            click.secho("Nothing... 😿", bold=True)
        else:
            click.echo(tabulate([x[1:] for x in self.data], headers=["UPDATED", "PULL REQUEST", "STATE"]))
            click.secho("Count: {}".format(len(self.data)), bold=True)


class OrganizationListQuery(GraphQLQuery):
    def __init__(self):
        super().__init__(S.ORGANIZATION_LIST_STATEMENT)
        self.data = []

    def _handle_response(self, response):
        orgas = response["data"]["viewer"]["organizations"]["edges"]
        for orga in orgas:
            login = orga["node"]["login"]
            url = orga["node"]["url"]
            self.data.append([login, url])

        self.data = sorted(self.data, key=lambda x: x[0].lower())

    def print(self):
        if len(self.data) == 0:
            click.secho("You do not belong to any organization 😿", bold=True)
        else:
            click.echo(tabulate(self.data, headers=["ORGANIZATION", "URL"]))
            click.secho("Count: {}".format(len(self.data)), bold=True)


class OrganizationMemberListQuery(GraphQLQuery):
    def __init__(self, organization):
        super().__init__(
            S.ORGANIZATION_MEMBER_LIST_STATEMENT, pagination_offset=100, extra_data={"organization": organization}
        )
        self.data = []
        self.organization = organization

    def get_pagination_info(self, response):
        return response["data"]["viewer"]["organization"]["membersWithRole"]["pageInfo"]

    def get_count(self):
        return len(self.data)

    def _handle_response(self, response):
        for member in response["data"]["viewer"]["organization"]["membersWithRole"]["edges"]:
            login = member["node"]["login"]
            email = member["node"]["email"]
            location = member["node"]["location"]
            role = member["role"]
            self.data.append([login, email, location, role])

        self.data = sorted(self.data, key=lambda x: x[0])

    def print(self):
        click.secho("{}'s members".format(self.organization), bold=True)
        click.echo(tabulate(self.data, headers=["NAME", "EMAIL", "LOCATION", "ROLE"]))
        click.secho("Count: {}".format(len(self.data)), bold=True)


class RateLimitQuery(GraphQLQuery):
    def __init__(self):
        super().__init__(S.RATE_LIMIT_STATEMENT)
        self.limit = None
        self.remaining = None
        self.reset_at = None

    def _handle_response(self, response):
        rate_limit = response["data"]["rateLimit"]
        self.limit = rate_limit["limit"]
        self.remaining = rate_limit["remaining"]
        self.reset_at = rate_limit["resetAt"]

    def print(self):
        click.secho("{}/{} until {}".format(self.remaining, self.limit, self.reset_at), bold=True)


class PullRequestListQuery(GraphQLQuery):
    def __init__(self):
        super().__init__(S.PULL_REQUEST_LIST_STATEMENT)
        self.data = []

    def _handle_response(self, response):
        for pr in response["data"]["viewer"]["pullRequests"]["edges"]:
            created = dt_for_str(pr["node"]["createdAt"]).date()
            url = pr["node"]["url"]
            title = shorten_str(pr["node"]["title"])
            mergeable = pr["node"]["mergeable"]
            created_str = days_ago_str(created)
            self.data.append([created, created_str, url, title, mergeable])
        # Sort by url, then by reversed date:
        self.data = sorted(self.data, key=lambda x: x[2])
        self.data = sorted(self.data, key=lambda x: x[0], reverse=True)

    def print(self):
        if len(self.data) == 0:
            click.secho("Nothing... 😿 Time to push hard 💪", bold=True)
        else:
            click.echo(tabulate([x[1:] for x in self.data], headers=["CREATED", "URL", "TITLE", "MERGEABLE"]))
            click.secho("Count: {}".format(len(self.data)), bold=True)


class OrgaPullRequestListQuery(GraphQLQuery):
    def __init__(self, organization):
        super().__init__(
            S.ORGA_PULL_REQUEST_LIST_STATEMENT, pagination_offset=10, extra_data={"organization": organization}
        )
        self.data = []

    def get_count(self):
        return len(self.data)

    def get_pagination_info(self, response):
        return response["data"]["search"]["pageInfo"]

    def _handle_response(self, response):
        for pr in response["data"]["search"]["edges"]:
            created = dt_for_str(pr["node"]["createdAt"]).date()
            url = pr["node"]["url"]
            title = shorten_str(pr["node"]["title"])
            created_str = days_ago_str(created)
            self.data.append([created, created_str, url, title])
        # Sort by url, then by reversed date:
        self.data = sorted(self.data, key=lambda x: x[2])
        self.data = sorted(self.data, key=lambda x: x[0], reverse=True)

    def print(self):
        if len(self.data) == 0:
            click.secho("Nothing... 😿 Time to push hard 💪", bold=True)
        else:
            click.echo(tabulate([x[1:] for x in self.data], headers=["CREATED", "URL", "TITLE"]))
            click.secho("Count: {}".format(len(self.data)), bold=True)


class ContributionListQuery:
    def __init__(self, dt_from, dt_to, organization=None):
        # TODO reengineer this class - create a generic one capable of executing
        #      queries sequentially
        self.pr_query = PullRequestContributionListQuery(dt_from, dt_to, organization)
        self.rv_query = PullRequestReviewContributionListQuery(dt_from, dt_to, organization)

    def execute(self):
        self.pr_query.execute()
        self.rv_query.execute()

    def print(self):
        data = []
        data.extend(self.pr_query.data)
        data.extend(self.rv_query.data)

        # Sort by url, then by reversed date:
        data = sorted(data, key=lambda x: (x[1], x[2], x[3]))
        data = sorted(data, key=lambda x: x[0], reverse=True)

        if len(data) == 0:
            click.secho("Nothing... 😿 Time to push hard 💪", bold=True)
        else:
            click.echo(tabulate(data, headers=["CREATED", "PULL REQUEST", "ROLE", "TITLE"]))
            click.secho("Count: {}".format(len(data)), bold=True)


class ContributionStatsQuery(GraphQLQuery):
    def __init__(self):
        super().__init__(S.CONTRIBUTION_STATS_STATEMENT)
        self.data = []

    def _handle_response(self, response):
        for k, v in response["data"]["viewer"]["contributionsCollection"].items():
            # `camelCase` to `Title case`
            key = "".join([" " + x.lower() if x.isupper() else x for x in k])
            key = key[0].upper() + key[1:]
            self.data.append([key, v])
        self.data = sorted(self.data, key=lambda x: x[0])

    def tabulate(self):
        return tabulate(self.data, headers=["STAT", "VALUE"])

    def print(self):
        click.echo(self.tabulate())


class PullRequestContributionListQuery(GraphQLQuery):
    def __init__(self, dt_from, dt_to, organization=None):
        super().__init__(
            S.PULL_REQUEST_CONTRIBUTION_LIST_STATEMENT,
            pagination_offset=100,
            extra_data={"organization": organization, "from": dt_from.isoformat(), "to": dt_to.isoformat()},
        )
        self.data = []

    def get_count(self):
        return len(self.data)

    def get_pagination_info(self, response):
        return response["data"]["viewer"]["contributionsCollection"]["pullRequestContributions"]["pageInfo"]

    def _handle_response(self, response):
        pr_contributions = response["data"]["viewer"]["contributionsCollection"]["pullRequestContributions"]["edges"]

        for pr_contribution in pr_contributions:
            created = dt_for_str(pr_contribution["node"]["pullRequest"]["createdAt"]).date()
            url = pr_contribution["node"]["pullRequest"]["url"]
            title = shorten_str(pr_contribution["node"]["pullRequest"]["title"])
            self.data.append([created, url, "OWNER", title])

        # Sort by url, then by reversed date:
        self.data = sorted(self.data, key=lambda x: (x[1], x[2], x[3]))
        self.data = sorted(self.data, key=lambda x: x[0], reverse=True)

    def tabulate(self):
        return tabulate(self.data, headers=["CREATED", "PULL REQUEST", "ROLE", "TITLE"])

    def print(self):
        if len(self.data) == 0:
            click.secho("Nothing... 😿 Time to push hard 💪", bold=True)
        else:
            click.echo(self.tabulate())
            click.secho("Count: {}".format(len(self.data)), bold=True)


class PullRequestReviewContributionListQuery(GraphQLQuery):
    def __init__(self, dt_from, dt_to, organization=None):
        super().__init__(
            S.PULL_REQUEST_REVIEW_CONTRIBUTION_LIST_STATEMENT,
            pagination_offset=100,
            extra_data={"organization": organization, "from": dt_from.isoformat(), "to": dt_to.isoformat()},
        )
        self.data = []

    def get_count(self):
        return len(self.data)

    def get_pagination_info(self, response):
        return response["data"]["viewer"]["contributionsCollection"]["pullRequestReviewContributions"]["pageInfo"]

    def _handle_response(self, response):
        rv_contributions = response["data"]["viewer"]["contributionsCollection"]["pullRequestReviewContributions"][
            "edges"
        ]

        for rv_contribution in rv_contributions:
            created = dt_for_str(rv_contribution["node"]["pullRequestReview"]["publishedAt"]).date()
            url = rv_contribution["node"]["pullRequest"]["url"]
            title = shorten_str(rv_contribution["node"]["pullRequest"]["title"])
            self.data.append([created, url, "REVIEWER", title])

        # Sort by url, then by reversed date:
        self.data = sorted(self.data, key=lambda x: (x[1], x[2], x[3]))
        self.data = sorted(self.data, key=lambda x: x[0], reverse=True)

    def tabulate(self):
        return tabulate(self.data, headers=["CREATED", "PULL REQUEST", "ROLE", "TITLE"])

    def print(self):
        if len(self.data) == 0:
            click.secho("Nothing... 😿 Time to push hard 💪", bold=True)
        else:
            click.echo(self.tabulate())
            click.secho("Count: {}".format(len(self.data)), bold=True)


class OneDayContributionListQuery(GraphQLQuery):
    def __init__(self, report_dt):
        super().__init__(S.ONE_DAY_CONTRIBUTION_LIST_STATEMENT, [], extra_data={"date": report_dt.isoformat()})
        self.data = []

    def _handle_response(self, response):
        pr_contributions = response["data"]["viewer"]["contributionsCollection"]["pullRequestContributions"]["edges"]
        rv_contributions = response["data"]["viewer"]["contributionsCollection"]["pullRequestReviewContributions"][
            "edges"
        ]

        for pr_contribution in pr_contributions:
            url = pr_contribution["node"]["pullRequest"]["url"]
            state = pr_contribution["node"]["pullRequest"]["state"]
            self.data.append([url, "OWNER", state])

        for rv_contribution in rv_contributions:
            url = rv_contribution["node"]["pullRequest"]["url"]
            state = rv_contribution["node"]["pullRequestReview"]["state"]
            self.data.append([url, "REVIEWER", state])

        self.data = sorted(self.data, key=lambda x: (x[0], x[1], x[2]))

    def tabulate(self):
        return tabulate(self.data, headers=["PULL REQUEST", "ROLE", "STATE"])

    def print(self):
        click.echo(self.tabulate())


class BranchListQuery(GraphQLQuery):
    def __init__(self, emails=None, is_dangling=False):
        super().__init__(S.BRANCH_LIST_STATEMENT, pagination_offset=10)
        self.data = []
        self.emails = emails
        self.is_dangling = is_dangling

    def get_pagination_info(self, response):
        return response["data"]["viewer"]["repositoriesContributedTo"]["pageInfo"]

    def get_count(self):
        return len(self.data)

    def _handle_response(self, response):
        for repo in response["data"]["viewer"]["repositoriesContributedTo"]["edges"]:
            repo_url = repo["node"]["url"]
            for branch in repo["node"]["refs"]["edges"]:
                branch_name = branch["node"]["name"]
                author_email = branch["node"]["target"]["author"]["email"]
                pr_list = []
                for pr in branch["node"]["associatedPullRequests"]["edges"]:
                    pr_list.append(pr["node"]["url"])
                if self.is_dangling and pr_list:
                    continue
                pr_list = sorted(pr_list)
                if self.emails is not None:
                    if author_email in self.emails:
                        if self.is_dangling:
                            self.data.append([repo_url, branch_name])
                        else:
                            self.data.append([repo_url, branch_name, "\n".join(pr_list)])

        self.data = sorted(self.data, key=lambda x: (x[0], x[1]))

    def print(self):
        no_results_message = "Nothing... 😿 Time to push hard 💪"
        headers = ["REPO", "BRANCH", "PULL REQUEST"]
        if self.is_dangling:
            no_results_message = "Everything is clean 👏"
            headers = ["REPO", "BRANCH"]

        if len(self.data) == 0:
            click.secho(no_results_message, bold=True)
        else:
            click.echo(tabulate(self.data, headers=headers))
            click.secho("Count: {}".format(len(self.data)), bold=True)


class EmailQuery(RESTQuery):
    def __init__(self):
        super().__init__("/user/emails")
        self.emails = None

    def _handle_response(self, response):
        self.emails = [x["email"] for x in response]

    def get_emails(self):
        return self.emails



"""
Subcommand `scrum`
"""
from datetime import datetime

import click

from yogit.yogit.scrum_report import generate_scrum_report
from yogit.yogit.checks import account_required, check_update
from yogit.utils.dateutils import today_str


def _compute_date_str(str_date):
    try:
        return datetime.strptime(str_date, "%Y-%m-%d")
    except ValueError:
        raise click.ClickException("Bad date format, should be `%Y-%m-%d`")


@click.group("scrum")
def scrum():
    """
    SCRUM actions
    """


@click.command("report", help="Generate your daily activity report")
@click.option(
    "--date", "str_date", type=click.STRING, help="Date of the report", default=today_str(), show_default=True
)
@click.pass_context
@account_required
@check_update
def scrum_report(ctx, str_date):  # pylint: disable=unused-argument
    """
    Generate your daily activity report
    """
    report_dt = _compute_date_str(str_date)
    generate_scrum_report(report_dt)


scrum.add_command(scrum_report)



""" Date utility functions """
from datetime import datetime


def _utcnow():
    return datetime.utcnow()


def dt_for_str(dt_string):
    """ Return datetime object from ISO 8601 string """
    return datetime.strptime(dt_string, r"%Y-%m-%dT%H:%M:%SZ")


def utcnow_str():
    """ Return ISO 8601 string of now instant """
    return _utcnow().replace(microsecond=0).isoformat()


def today_str():
    """ Return ISO 8601 string of today """
    return _utcnow().strftime(r"%Y-%m-%d")


def today_earliest_str():
    """ Return ISO 8601 string of the begining of today """
    return _utcnow().replace(hour=0, minute=0, second=0, microsecond=0).isoformat()


def days_ago_str(dte):
    """ Return human readable string between today and another date object """
    today = _utcnow().date()
    delta_days = (today - dte).days
    if delta_days == 0:
        return "Today"
    if delta_days == 1:
        return "Yesterday"
    return "{} days ago".format(delta_days)



"""
yogit storage
"""

import os
import yaml

from yogit.yogit.logger import LOGGER


class Storage:
    """ Storage based on YAML file """

    def __init__(self, filename, version=None):
        self.filename = filename
        self.version = version

    def get_path(self):
        """
        Get storage file path
        """
        return self.filename

    def load(self):
        """ Load YAML """
        try:
            with open(self.filename, "r") as yaml_file:
                return yaml.load(yaml_file, Loader=yaml.FullLoader) or {}
        except Exception as error:
            LOGGER.error(str(error))
            return {}

    def save(self, data):
        with open(self.filename, "w") as yaml_file:
            if data is not None and self.version is not None:
                data["version"] = self.version
            yaml.safe_dump(data, stream=yaml_file, indent=4)

    def get_version(self):
        data = self.storage.load()
        return data.get("version", None) or None



"""
Subcommand `ct`
"""
from datetime import datetime

import click

from yogit.api.queries import ContributionListQuery, ContributionStatsQuery
from yogit.yogit.checks import account_required, check_update
from yogit.utils.dateutils import today_str


def _compute_date_str(str_from, str_to):
    try:
        dt_from = datetime.strptime(str_from, "%Y-%m-%d")
        dt_to = datetime.strptime(str_to, "%Y-%m-%d")
        dt_to = dt_to.replace(hour=23, minute=59, second=59, microsecond=0)
    except ValueError:
        raise click.ClickException("Bad date format, should be `%Y-%m-%d`")
    if dt_from > dt_to:
        raise click.ClickException("`--from` is not before `--to`")
    return dt_from, dt_to


def _get_default_to():
    return today_str()


def _get_default_from():
    return today_str()


@click.group("contrib")
def contribution():
    """
    Contribution actions
    """


@click.command("list", help="List your contributions")
@click.option(
    "--to",
    "str_to",
    type=click.STRING,
    help="End search at this date (included)",
    default=_get_default_to(),
    show_default=True,
)
@click.option(
    "--from",
    "str_from",
    type=click.STRING,
    help="Begin search at this date",
    default=_get_default_from(),
    show_default=True,
)
@click.pass_context
@account_required
@check_update
def contribution_list(ctx, str_from, str_to):  # pylint: disable=unused-argument
    """
    List contributions
    """
    dt_from, dt_to = _compute_date_str(str_from, str_to)
    if (dt_to - dt_from).days > 365:
        raise click.ClickException("Date range must not exceed one year")
    click.secho("Contributions from {} to {}".format(str_from, str_to), bold=True)
    query = ContributionListQuery(dt_from, dt_to)
    query.execute()  # pylint: disable=no-value-for-parameter
    query.print()


@click.command("stats", help="GitHub statistics")
@click.pass_context
@account_required
@check_update
def contribution_stats(ctx):  # pylint: disable=unused-argument
    """
    List GitHub statistics
    """
    query = ContributionStatsQuery()
    query.execute()  # pylint: disable=no-value-for-parameter
    query.print()


contribution.add_command(contribution_list)
contribution.add_command(contribution_stats)



"""
GitHub API requester
"""
import json
import requests

from requests_toolbelt.utils import dump
import click

from yogit.yogit.logger import LOGGER
from yogit.yogit.settings import Settings

GITHUB_API_URL_V4 = "https://api.github.com/graphql"
GITHUB_API_URL_V3 = "https://api.github.com"


def _http_call(method, url, **kwargs):
    """
    Perform HTTP call and log around it
    """
    try:
        response = requests.request(method, url, **kwargs)
        if response:
            LOGGER.info("Response: %s", response.status_code)
        else:
            LOGGER.info("Response: %s", dump.dump_all(response).decode("utf-8"))
        return response
    except requests.RequestException as exception:
        LOGGER.error(str(exception))
        raise click.ClickException(str(exception))


def _get_authorization():
    """
    Craft Authorization HTTP header
    """
    token = Settings().get_token()
    return "token {}".format(token)


def _get_headers():
    """
    Craft HTTP headers for request
    """
    return {"Accept": "application/json", "Content-Type": "application/json", "Authorization": _get_authorization()}


class GraphQLClient:
    """
    GitHub GraphQL API client
    """

    def __init__(self):
        self.url = GITHUB_API_URL_V4

    def get(self, query):
        """
        Perform GET GitHub GraphQL request
        """
        payload = json.dumps({"query": query})
        LOGGER.debug(payload)
        response = _http_call("post", self.url, headers=_get_headers(), data=payload)
        LOGGER.debug(response.content[:500])
        if response.status_code == 200:
            try:
                return response.json()
            except Exception as exception:
                LOGGER.error(str(exception))
                raise click.ClickException(response.text)
        elif response.status_code == 400:
            raise click.ClickException("Bad request")
        elif response.status_code == 401:
            raise click.ClickException("Unauthorized")
        else:
            raise click.ClickException(response.text)


class RESTClient:
    """
    GitHub REST API client
    """

    def _get_url(self, endpoint):
        return GITHUB_API_URL_V3 + endpoint

    def get(self, endpoint):
        """
        Perform GET GitHub REST request
        """
        url = self._get_url(endpoint)
        LOGGER.debug("GET %s", url)
        response = _http_call("get", url, headers=_get_headers())
        LOGGER.debug(response.content[:500])
        if response.status_code == 200:
            try:
                return response.json()
            except Exception as exception:
                LOGGER.error(str(exception))
                raise click.ClickException(response.text)
        elif response.status_code == 400:
            raise click.ClickException("Bad request")
        elif response.status_code == 401:
            raise click.ClickException("Unauthorized")
        else:
            raise click.ClickException(response.text)



"""
Subcommand `orga`
"""
import random
from time import sleep

import click

from yogit.api.queries import OrganizationListQuery, OrganizationMemberListQuery
from yogit.yogit.checks import account_required, check_update
from yogit.utils.spinner import get_spinner_object


def check_organization(orga):
    query = OrganizationListQuery()
    query.execute()
    orgas = [x[0].lower() for x in query.data]
    if len(orgas) == 0:
        raise click.ClickException("You do not belong to any organization 😿")
    if orga is None:
        if len(orgas) == 1:
            return orgas[0]
        else:
            raise click.ClickException(
                "You belong to more than one organization (hint: `yogit orga list`), use `--orga` option to discriminate"
            )
    if orga.lower() not in orgas:
        raise click.ClickException("Unrecognized {} organization (hint: `yogit orga list`)".format(orga))
    return orga


@click.group("orga")
def organization():
    """
    Organization actions
    """


@click.command("list", help="List organizations you belong to")
@click.pass_context
@account_required
@check_update
def orga_list(ctx):  # pylint: disable=unused-argument
    """
    List organizations you belong to
    """
    query = OrganizationListQuery()
    query.execute()  # pylint: disable=no-value-for-parameter
    query.print()


@click.group("member")
def member():
    """
    Member actions
    """


@click.command("list", help="List members of the organization you belong to")
@click.option("--orga", type=click.STRING, help="Specify the organization")
@click.pass_context
@account_required
@check_update
def orga_member_list(ctx, orga):  # pylint: disable=unused-argument
    """
    List members of the organization you belong to
    """
    organization = check_organization(orga)
    query = OrganizationMemberListQuery(organization)
    query.execute()  # pylint: disable=no-value-for-parameter
    query.print()


@click.command("pickone", help="Randomly pick a member of the organization you belong to")
@click.option("--orga", type=click.STRING, help="Specify the organization")
@click.pass_context
@account_required
@check_update
def orga_member_pickone(ctx, orga):  # pylint: disable=unused-argument
    """
    Randomly pick a member of the organization you belong to
    """
    organization = check_organization(orga)
    query = OrganizationMemberListQuery(organization)
    query.execute()  # pylint: disable=no-value-for-parameter
    members = [x[0] for x in query.data]
    count = len(members)
    click.secho("Picking one out of {} members... ({:.2f}%) 🎲".format(count, 100 / count), bold=True)
    random.seed()
    with get_spinner_object() as spinner:
        for _ in range(20):  # Rolling for 5 seconds
            picked_member = members[random.randint(0, count - 1)]
            spinner.text = picked_member
            sleep(0.25)
    click.secho('The winner is "{}" 🤠'.format(picked_member), bold=True)


organization.add_command(orga_list)
organization.add_command(member)
member.add_command(orga_member_list)
member.add_command(orga_member_pickone)



"""
Subcommand `br`
"""

import click

from yogit.api.queries import BranchListQuery
from yogit.yogit.settings import Settings
from yogit.yogit.checks import account_required, check_update


@click.group("br")
def branch():
    """
    Branch actions
    """


@click.command("list", help="List your branches")
@click.option("--dangling", is_flag=True, help="Only show branches which does not have associated pull request")
@click.pass_context
@account_required
@check_update
def branch_list(ctx, dangling):  # pylint: disable=unused-argument
    """
    List your branches
    """
    query = BranchListQuery(emails=Settings().get_emails(), is_dangling=dangling)
    query.execute()  # pylint: disable=no-value-for-parameter
    query.print()


branch.add_command(branch_list)



"""
yogit settings
"""
import yaml

from yogit.yogit.paths import get_settings_path, get_scrum_report_path
from yogit.storage.storage import Storage

SETTINGS_VERSION = 1
SCRUM_REPORT_VERSION = 1
DEFAULT_SCRUM_REPORT_CONFIG = """
# Available placeholders:
# questions: list of question to ask
# template: report template, each element is a line, available placeholders are:
#   ${today}: "yyyy-MM-dd
#   ${qx}: x-th question
#   ${ax}: x-th answer
#   ${github_report}: GitHub activity presented in a table

questions:
- "What have you done today?"
- "Do you have any blockers?"
- "What do you plan to work on your next working day?"

template:
- "*REPORT ${today}*"
- "*${q0}*"
- "${a0}"
- "*${q1}*"
- "${a1}"
- "*${q2}*"
- "${a2}"
- ""
- "```"
- "${github_report}"
- "```"
"""


class Settings:
    """ Settings access class """

    def __init__(self):
        self.storage = Storage(get_settings_path(), SETTINGS_VERSION)

    def get_path(self):
        """
        Get storage file path
        """
        return self.storage.get_path()

    def reset(self):
        """ Reset setting values """
        self.storage.save(None)

    def is_valid(self):
        """ Return True if account is setup, False otherwise """
        return self.get_token() != "" and self.get_login() != "" and self.get_emails() != []

    def get_token(self):
        """ Return GitHub token or empty string """
        data = self.storage.load()
        return data.get("token", "") or ""

    def set_token(self, token):
        """ Store GitHub token """
        data = self.storage.load()
        data["token"] = token
        self.storage.save(data)

    def get_login(self):
        """ Return login identifier or empty string """
        data = self.storage.load()
        return data.get("login", "") or ""

    def set_login(self, login):
        """ Store login identifier """
        data = self.storage.load()
        data["login"] = login
        self.storage.save(data)

    def get_emails(self):
        """ Return email list associated to the GitHub account or empty list """
        data = self.storage.load()
        return data.get("emails", []) or []

    def set_emails(self, emails):
        """ Store email list """
        data = self.storage.load()
        data["emails"] = emails
        self.storage.save(data)


class ScrumReportSettings:
    """
    Scrum report settings access class
    """

    def __init__(self):
        self.storage = Storage(get_scrum_report_path(), SCRUM_REPORT_VERSION)

    def get_path(self):
        """
        Get storage file path
        """
        return self.storage.get_path()

    def get(self):
        """
        Return scrum report data

        If not present a default one is stored and returned
        """
        data = self.storage.load()
        if data == {}:
            data = yaml.load(DEFAULT_SCRUM_REPORT_CONFIG, Loader=yaml.FullLoader)
            self.storage.save(data)
        return data



"""
yogit precheck routines
"""
import click

from yogit.yogit.settings import Settings
from yogit.yogit.update_checker import UpdateChecker


def account_required(func):
    """
    Check if account setup has been performed
    """

    def wrapper(self, *args, **kwargs):
        # pylint: disable=missing-docstring
        if not Settings().is_valid():
            raise click.ClickException("Account required, please `yogit account setup` first.")
        func(self, *args, **kwargs)

    return wrapper


def _check_update():
    UpdateChecker().check()


def check_update(func):
    """
    Check if a new version of yogit is available
    """

    def wrapper(self, *args, **kwargs):
        # pylint: disable=missing-docstring
        _check_update()
        func(self, *args, **kwargs)

    return wrapper



"""
Subcommand `pr`
"""

import click

from yogit.api.queries import PullRequestListQuery, OrgaPullRequestListQuery
from yogit.yogit.checks import account_required, check_update


@click.group("pr")
def pull_request():
    """
    Pull request actions
    """


@click.command("list", help="List your opened pull requests")
@click.option("--orga", type=click.STRING, help="Expand results to a specific organization")
@click.pass_context
@account_required
@check_update
def pull_request_list(ctx, orga):  # pylint: disable=unused-argument
    """
    List pull requests
    """
    if orga:
        query = OrgaPullRequestListQuery(orga)
    else:
        query = PullRequestListQuery()
    query.execute()  # pylint: disable=no-value-for-parameter
    query.print()


pull_request.add_command(pull_request_list)



"""
yogit paths
"""
import os

SETTINGS_DIR = os.path.join(os.path.expanduser("~/.yogit"))


def get_log_path():
    """ Get yogit log path """
    return os.path.join(SETTINGS_DIR, "yogit.log")


def get_settings_path():
    """ Get yogit config path """
    return os.path.join(SETTINGS_DIR, "config.yaml")


def get_scrum_report_path():
    """ Get scrum report path """
    return os.path.join(SETTINGS_DIR, "scrum_report.yaml")



"""
GraphQL requesters used by yogit
"""
from string import Template

from yogit.yogit.settings import Settings
import yogit.api.statements as S
from yogit.utils.dateutils import today_earliest_str


def prepare(statement, variables, extra_data={}):
    """
    Set variable in statement and return the prepared statement
    """
    template = Template(statement)
    data = extra_data.copy()
    for variable in variables:
        if variable == S.LOGIN_VARIABLE:
            data[variable] = Settings().get_login()
        elif variable == S.TODAY_VARIABLE:
            data[variable] = today_earliest_str()
    return template.safe_substitute(data)


def prepare_pagination(statement, offset, cursor=None):
    template = Template(statement)
    data = {"offset": offset, "after": 'after: "{}"'.format(cursor) if cursor is not None else ""}
    return template.safe_substitute(data)



"""
yogit entry point
"""
import click

from yogit import get_name, get_version
from yogit.yogit.logger import enable_stdout
from yogit.yogit.organization import organization
from yogit.yogit.pullrequest import pull_request
from yogit.yogit.branch import branch
from yogit.yogit.contribution import contribution
from yogit.yogit.review import review
from yogit.yogit.account import account
from yogit.yogit.scrum import scrum


def get_version_content():
    """
    Return content printed by `yogit --version` command
    """
    return ("{}, version {}\n" "Changelog: https://github.com/hasboeuf/yogit/blob/{}/CHANGELOG.md").format(
        get_name(), get_version(), get_version()
    )


@click.group()
@click.version_option(message=get_version_content())
@click.option("--verbose", "-v", is_flag=True, help="Print verbose output.")
@click.pass_context
def main(ctx, verbose):  # pylint: disable=unused-argument
    """
    Command line utility for GitHub daily work
    """
    if verbose:
        enable_stdout()


main.add_command(account)
main.add_command(branch)
main.add_command(contribution)
main.add_command(organization)
main.add_command(pull_request)
main.add_command(review)
main.add_command(scrum)


if __name__ == "__main__":
    main()  # pylint: disable=no-value-for-parameter



"""
`scrum report` command logic
"""

import re
from string import Template

import click
import pyperclip

from yogit.yogit.settings import ScrumReportSettings
from yogit.api.queries import OneDayContributionListQuery
from yogit.yogit.logger import LOGGER


def _get_github_report(report_dt):
    try:
        query = OneDayContributionListQuery(report_dt)
        query.execute()  # pylint: disable=no-value-for-parameter
        return query.tabulate()
    except Exception as exception:
        LOGGER.error(str(exception))
        return str(exception)


def generate_scrum_report(report_dt):
    """
    Generate scrum report based on scrum report template

    Also copy the report in clipboard if wanted
    """
    settings = ScrumReportSettings()
    settings_data = settings.get()
    click.secho("Tips:", bold=True)
    click.echo("• To customize report template, edit `{}`".format(settings.get_path()))
    click.echo("• Begin line with an extra " + click.style("<space>", bold=True) + " to indent it")
    click.echo("")
    click.secho("Report of {}".format(report_dt.date().isoformat()), bold=True)

    data = {}
    try:
        questions = settings_data["questions"]
        tpl = settings_data["template"]
    except Exception as error:
        LOGGER.error(str(error))
        raise click.ClickException("Unable to parse SCRUM report template")

    suffix = "• "
    for idx, question in enumerate(questions):
        click.echo(click.style(question, bold=True) + " (empty line to move on)")
        answers = []
        while True:
            line = click.prompt("", prompt_suffix=suffix, default="", show_default=False)
            if line == "":
                break
            line = suffix + line
            line = re.sub("^•  ", "    ‣ ", line)
            answers.append(line)
        data["q{}".format(idx)] = question
        data["a{}".format(idx)] = "\n".join(answers)

    template = Template("\n".join(tpl))

    data["today"] = report_dt.date().isoformat()  # "today" string does is not meaninful
    if "${github_report}" in template.template:
        data["github_report"] = _get_github_report(report_dt)

    report = template.safe_substitute(data)

    if click.confirm("Copy to clipboard?", prompt_suffix=" "):
        try:
            pyperclip.copy(report)
            click.secho("Copied! 🤘", bold=True)
        except Exception as error:
            click.echo(report)
            LOGGER.error(str(error))
            raise click.ClickException("Not supported on your system, please `sudo apt-get install xclip`")
    else:
        click.echo(report)



"""
Command line utility for GitHub daily work.
"""
import setuptools
import yogit

DEPENDENCIES = ["click", "halo", "packaging", "PyYAML>=5.1", "pyperclip", "requests", "requests-toolbelt", "tabulate"]


def get_public_description():
    with open("README.md", encoding="utf-8") as md_file:
        package_description_content = md_file.read()
    with open("CHANGELOG.md", encoding="utf-8") as md_file:
        changelog_content = md_file.read()
    return package_description_content + "\n\n" + changelog_content


setuptools.setup(
    name=yogit.__application__,
    version=yogit.__version__,
    author="Adrien Gavignet",
    author_email="adrien.gavignet@gmail.com",
    license="MIT",
    description="Command line utility for GitHub daily work.",
    long_description=get_public_description(),
    long_description_content_type="text/markdown",
    keywords="git github utility branch pull requests",
    url="https://github.com/hasboeuf/yogit",
    packages=setuptools.find_packages(exclude=["*.tests", "*.tests.*", "tests"]),
    classifiers=["Programming Language :: Python :: 3", "Operating System :: OS Independent"],
    zip_safe=True,
    install_requires=DEPENDENCIES,
    entry_points={"console_scripts": ["yogit=yogit.yogit.cli:main"]},
)



"""
GraphQL query statements used by yogit
"""
LOGIN_VARIABLE = "login"
TODAY_VARIABLE = "today"

LOGIN_STATEMENT = """
{
    viewer {
        login
    }
}"""

ORGANIZATION_LIST_STATEMENT = """
{
    viewer {
        organizations(last: 100) {
            edges {
                node {
                    login
                    url
                }
            }
        }
    }
}
"""


REVIEW_REQUESTED_STATEMENT = """
{
    search(query: "type:pr state:open review-requested:$login", type: ISSUE, first: 100) {
        edges {
            node {
                ... on PullRequest {
                    repository {
                        nameWithOwner
                    }
                    number
                    url
                }
            }
        }
    }
}"""


REVIEW_LIST_STATEMENT = """
query {
    viewer {
        contributionsCollection {
            pullRequestReviewContributions(first: $offset $after) {
                pageInfo {
                    hasNextPage
                    endCursor
                }
                edges {
                    node {
                        pullRequestReview {
                            createdAt
                            updatedAt
                            state
                        }
                        pullRequest {
                            url
                            state
                            commits(last:1) {
                                edges {
                                    node {
                                        commit {
                                            pushedDate
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}"""

ORGANIZATION_MEMBER_LIST_STATEMENT = """
query {
    viewer {
        organization(login: $organization) {
            membersWithRole(first: $offset $after) {
                pageInfo {
                    hasNextPage
                    endCursor
                }
                totalCount
                edges {
                    role
                    node {
                        login
                        email
                        location
                    }
                }
            }
        }
    }
}
"""

RATE_LIMIT_STATEMENT = """
{
    rateLimit {
        limit
        cost
        remaining
        resetAt
    }
}
"""


PULL_REQUEST_LIST_STATEMENT = """
{
    viewer {
        pullRequests(first:100, states: OPEN) {
            edges {
                node {
                    createdAt
                    url
                    title
                    mergeable
                }
            }
        }
    }
}
"""

ORGA_PULL_REQUEST_LIST_STATEMENT = """
{
    search(query: "is:open is:pr archived:false user:$organization", type: ISSUE, first: $offset $after) {
        pageInfo {
            hasNextPage
            endCursor
        }
        edges {
            node {
                ... on PullRequest {
                    repository {
                        nameWithOwner
                    }
                    createdAt
                    number
                    url
                    title
                }
            }
        }
    }
}
"""

ONE_DAY_CONTRIBUTION_LIST_STATEMENT = """
{
    viewer {
        contributionsCollection(from: "$date", to: "$date") {
            pullRequestContributions(first: 100) {
                edges {
                    node {
                        pullRequest {
                            url
                            title
                            state
                            createdAt
                            updatedAt
                        }
                    }
                }
            }
            pullRequestReviewContributions(first: 100) {
                edges {
                    node {
                        pullRequestReview {
                            url
                            publishedAt
                            state
                        }
                        pullRequest {
                            url
                        }
                    }
                }
            }
        }
    }
}
"""

PULL_REQUEST_CONTRIBUTION_LIST_STATEMENT = """
{
    viewer {
        contributionsCollection(from: "$from", to: "$to") {
            pullRequestContributions(first: $offset $after) {
                pageInfo {
                    hasNextPage
                    endCursor
                }
                edges {
                    node {
                        pullRequest {
                            url
                            title
                            createdAt
                        }
                    }
                }
            }
        }
    }
}
"""

PULL_REQUEST_REVIEW_CONTRIBUTION_LIST_STATEMENT = """
{
    viewer {
        contributionsCollection(from: "$from", to: "$to") {
            pullRequestReviewContributions(first: $offset $after) {
                pageInfo {
                    hasNextPage
                    endCursor
                }
                edges {
                    node {
                        pullRequestReview {
                            publishedAt
                        }
                        pullRequest {
                            url
                            title
                        }
                    }
                }
            }
        }
    }
}
"""

CONTRIBUTION_STATS_STATEMENT = """
{
    viewer {
        contributionsCollection {
            totalIssueContributions
            totalCommitContributions
            totalRepositoryContributions
            totalPullRequestContributions
            totalPullRequestReviewContributions
            totalRepositoriesWithContributedIssues
            totalRepositoriesWithContributedCommits
            totalRepositoriesWithContributedPullRequests
            totalRepositoriesWithContributedPullRequestReviews
        }
    }
}
"""

BRANCH_LIST_STATEMENT = """
{
    viewer {
        repositoriesContributedTo(first: $offset $after) {
            pageInfo {
                hasNextPage,
                endCursor
            },
            edges {
                node {
                    url
                    refs(first: 100, refPrefix: "refs/heads/") {
                        edges {
                            node {
                                associatedPullRequests(first: 10) {
                                    edges {
                                        node {
                                            url
                                            headRefName
                                        }
                                    }
                                }
                                name
                                target {
                                    ... on Commit {
                                        author {
                                            email
                                            name
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}
"""



"""
Subcommand `rv`
"""

import click

from yogit.api.queries import ReviewListQuery, ReviewRequestedQuery
from yogit.yogit.checks import account_required, check_update


@click.group("rv")
def review():
    """
    Review actions
    """


@click.command("list", help="List your reviews on opened pull requests")
@click.pass_context
@account_required
@check_update
def review_list(ctx):  # pylint: disable=unused-argument
    """
    List your reviews on opened pull requests
    """
    query = ReviewListQuery()
    query.execute()  # pylint: disable=no-value-for-parameter
    query.print()


@click.command("requested", help="List pull requests where your review is requested")
@click.pass_context
@account_required
@check_update
def review_requested_list(ctx):  # pylint: disable=unused-argument
    """
    List pull requests where your review is requested
    """
    query = ReviewRequestedQuery()
    query.execute()  # pylint: disable=no-value-for-parameter
    query.print()


review.add_command(review_list)
review.add_command(review_requested_list)



"""
Account commands
"""

import click

from yogit.api.queries import LoginQuery, EmailQuery, RateLimitQuery
from yogit.yogit.settings import Settings
from yogit.yogit.checks import account_required, check_update


def get_welcome_text():
    """
    Return Welcome text
    """
    settings = Settings()
    return r"""                   _ _
 _   _  ___   __ _(_) |_ 
| | | |/ _ \ / _` | | __|
| |_| | (_) | (_| | | |_ 
 \__, |\___/ \__, |_|\__|
 |___/       |___/       

Welcome to yogit!

Go here to generate a GitHub personal access token:
https://github.com/settings/tokens
Required scopes are:
• read:org
• read:user
• user:email
• repo

Configuration is stored here `{}`,
and is not encrypted, only use yogit on your personal computer.
""".format(
        settings.get_path()
    )


@click.group("account")
def account():
    """
    Account actions
    """


@click.command("setup", help="Setup yogit")
def account_setup():
    """
    Setup yogit
    """
    settings = Settings()
    settings.reset()

    click.echo(get_welcome_text())
    token = click.prompt("GitHub token", type=click.STRING, hide_input=True).strip()

    settings.set_token(token)

    try:
        login_query = LoginQuery()
        login_query.execute()  # pylint: disable=no-value-for-parameter
        login = login_query.get_login()

        email_query = EmailQuery()
        email_query.execute()  # pylint: disable=no-value-for-parameter
    except Exception as exception:
        settings.reset()
        raise exception

    settings.set_login(login)
    settings.set_emails(email_query.get_emails())

    click.secho("Hello {}! 💕✨".format(login), bold=True)


@click.command("usage", help="Account API usage")
@click.pass_context
@account_required
@check_update
def account_usage(ctx):  # pylint: disable=unused-argument
    """
    Account API usage
    """
    query = RateLimitQuery()
    query.execute()  # pylint: disable=no-value-for-parameter
    query.print()


account.add_command(account_setup)
account.add_command(account_usage)
